import React, { useEffect, useState } from "react";
import { styled } from "@mui/material/styles";
import {
  Button,
  TextField,
  Dialog,
  DialogActions,
  DialogContent,
  DialogTitle,
  CircularProgress,
  Autocomplete,
  Alert
} from "@mui/material";
import { useNamingNames, useAllowedGitProjects } from "../../../api/SwaggerApi";
import { useTypingTimer } from "../../common/SearchBoxFilter/TypingTimer";

const PREFIX = "IOCCreateDialog";

const classes = {
  textField: `${PREFIX}-textField`
};

const StyledDialog = styled(Dialog)(({ theme }) => ({
  [`& .${classes.textField}`]: {
    marginBottom: theme.spacing(1)
  }
}));

export function IOCCreateDialog({
  open,
  setOpen,
  submitCallback,
  error,
  resetError
}) {
  const [names, getNames, , loadingNames] = useNamingNames();
  const [name, setName] = useState();
  const [nameQuery, onNameKeyUp] = useTypingTimer({ interval: 500 });
  const [
    allowedGitProjects,
    getAllowedGitProjects,
    ,
    loadingAllowedGitProjects
  ] = useAllowedGitProjects([]);
  const [gitId, setGitId] = useState(null);

  const handleClose = () => {
    setOpen(false);
  };

  useEffect(() => getNames(nameQuery), [nameQuery, getNames]);

  useEffect(() => {
    getAllowedGitProjects();
  }, [getAllowedGitProjects]);

  const onSubmit = (event) => {
    event.preventDefault();

    submitCallback({
      gitProjectId: gitId,
      namingUuid: name ? name.uuid : undefined
    });
  };

  return (
    <StyledDialog
      open={open}
      onClose={handleClose}
      fullWidth
      maxWidth="sm"
    >
      <form onSubmit={onSubmit}>
        <DialogTitle
          id="form-dialog-title"
          data-testid="create-dialog-title"
        >
          Create new IOC
        </DialogTitle>
        <DialogContent>
          <Autocomplete
            className={classes.textField}
            autoHighlight
            id="nameAutocomplete"
            options={names}
            loading={loadingNames}
            clearOnBlur={false}
            getOptionLabel={(option) => {
              return option?.name;
            }}
            renderInput={(params) => (
              <TextField
                {...params}
                label="IOC name"
                variant="outlined"
                required
                InputProps={{
                  ...params.InputProps,
                  endAdornment: (
                    <React.Fragment>
                      {loadingNames ? (
                        <CircularProgress
                          color="inherit"
                          size={20}
                        />
                      ) : null}
                      {params.InputProps.endAdornment}
                    </React.Fragment>
                  )
                }}
              />
            )}
            onChange={(event, value, reason) => {
              setName(value);
              resetError();
            }}
            onInputChange={(event, value, reason) => {
              event && onNameKeyUp(event.nativeEvent);
            }}
            autoSelect
          />

          <TextField
            autoComplete="off"
            className={classes.textField}
            id="description"
            label="--- IOC Description from Naming ---"
            variant="standard"
            fullWidth
            disabled
            value={name ? name.description : ""}
          />

          <Autocomplete
            className={classes.textField}
            autoHighlight
            id="gitId"
            options={allowedGitProjects}
            loading={loadingAllowedGitProjects}
            clearOnBlur={false}
            defaultValue={{
              id: null,
              url: ""
            }}
            getOptionLabel={(option) => {
              return option.url;
            }}
            onChange={(event, value, reason) => {
              setGitId(value?.id);
              resetError();
            }}
            renderInput={(params) => (
              <TextField
                {...params}
                label="Git repository"
                variant="outlined"
                fullWidth
                required
                InputProps={{
                  ...params.InputProps,
                  endAdornment: (
                    <React.Fragment>
                      {loadingAllowedGitProjects ? (
                        <CircularProgress
                          color="inherit"
                          size={20}
                        />
                      ) : null}
                      {params.InputProps.endAdornment}
                    </React.Fragment>
                  )
                }}
              />
            )}
            autoSelect
          />

          {error ? <Alert severity="error">{error}</Alert> : <></>}
        </DialogContent>
        <DialogActions>
          <Button
            onClick={handleClose}
            color="primary"
          >
            Cancel
          </Button>
          <Button
            color="primary"
            variant="contained"
            type="submit"
            disabled={!name || !gitId}
          >
            Create
          </Button>
        </DialogActions>
      </form>
    </StyledDialog>
  );
}
