import React, { useEffect, useState, useMemo } from "react";
import { styled } from "@mui/material/styles";
import {
  Grid,
  Typography,
  Card,
  CardContent,
  Container,
  Link as MuiLink
} from "@mui/material";
import { DeploymentStepper } from "../deployments/DeploymentStepper";
import { KeyValueTable } from "@ess-ics/ce-ui-common/dist/components/common/KeyValueTable";
import { SimpleAccordion } from "../common/Accordion/SimpleAccordion";
import { JobBadge } from "./JobBadge";
import { DeploymentJobOutput } from "../deployments/DeploymentJobOutput";
import { Link as ReactRouterLink } from "react-router-dom";
import { formatDate } from "../common/Helper";
import GitRefLink from "../IOC/GitRefLink";
import AccessControl from "../auth/AccessControl";
import { AWXJobDetails } from "../../api/DataTypes";
import AlertMessages from "../IOC/AlertMessages";

const PREFIX = "JobDetails";

const classes = {
  csentryIdInvalid: `${PREFIX}-csentryIdInvalid`
};

const StyledGrid = styled(Grid)(({ theme }) => ({
  [`& .${classes.csentryIdInvalid}`]: {
    color: theme.csentryIdInvalid.color
  }
}));

function createAlert(operation, job) {
  const jobDetails = new AWXJobDetails(operation.type, job);
  const message = jobDetails.message();
  const severity = jobDetails.severity();
  return {
    type: severity,
    message: message
  };
}

export function JobDetails({ operation, job }) {
  let jobAlert = useMemo(() => operation.alerts ?? [], [operation]);

  const finishedJob = useMemo(
    () => operation && new AWXJobDetails(operation.type, job).isFinished(),
    [operation, job]
  );

  const [alert, setAlert] = useState(createAlert(operation, job));

  const jobDetails = useMemo(
    () => new AWXJobDetails(operation.type, job),
    [operation, job]
  );

  useEffect(() => {
    setAlert(createAlert(operation, job, finishedJob));
  }, [finishedJob, operation, job]);

  function calculateHostText() {
    // host is resolvable => show link for users
    if (operation.host.csEntryId && operation.host.csEntryIdValid) {
      return (
        <Typography>
          <MuiLink
            component={ReactRouterLink}
            to={`/hosts/${operation.host.csEntryId}`}
            underline="hover"
          >
            {operation.host.fqdn}
          </MuiLink>
        </Typography>
      );
    }

    // gray out hostname if it is from cache
    return (
      <Typography className={classes.csentryIdInvalid}>
        {operation.host.fqdn}
      </Typography>
    );
  }

  const deploymentDetails = {
    Revision: (
      <GitRefLink
        url={operation.gitProjectUrl}
        revision={operation.gitShortReference}
      />
    ),
    comment: operation.comment ? operation.comment : "",
    host: calculateHostText(),
    user: operation.createdBy,
    repository: (
      <Typography>
        <MuiLink
          href={operation.gitProjectUrl}
          target="_blank"
          rel="noreferrer"
          underline="hover"
        >
          {operation.gitProjectUrl}
        </MuiLink>
      </Typography>
    ),
    "AWX job link": (
      <Typography>
        <MuiLink
          href={operation.awxJobUrl}
          target="_blank"
          rel="noreferrer"
          underline="hover"
        >
          {operation.awxJobUrl}
        </MuiLink>
      </Typography>
    ),
    "created time": operation?.createdAt
      ? formatDate(operation.createdAt)
      : "-",
    "AWX job start time": operation?.startTime
      ? formatDate(operation.startTime)
      : "-"
  };

  return (
    <StyledGrid
      container
      spacing={1}
    >
      {finishedJob && (
        <Grid
          item
          xs={12}
        >
          <AlertMessages alerts={[alert].concat(jobAlert)} />
        </Grid>
      )}
      <Grid
        item
        xs={12}
        md={12}
      >
        <SimpleAccordion
          defaultExpanded
          summary={<JobBadge operation={operation} />}
        >
          <KeyValueTable
            obj={deploymentDetails}
            variant="overline"
            sx={{ border: 0 }}
            valueOptions={{ headerName: "" }}
          />
        </SimpleAccordion>
      </Grid>
      <Grid
        item
        xs={12}
        md={12}
      >
        <Card>
          <CardContent>
            <div>
              {job && (
                <DeploymentStepper
                  activeStep={["queued", "running", "successful"].indexOf(
                    job.status.toLowerCase()
                  )}
                  deploymentStart={operation.startDate}
                  jobDetails={jobDetails}
                />
              )}
            </div>
          </CardContent>
        </Card>
      </Grid>
      {!finishedJob && alert && (
        <Grid
          item
          xs={12}
        >
          <AlertMessages alerts={[alert]} />
        </Grid>
      )}
      <AccessControl
        allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
        renderNoAccess={() => <></>}
      >
        <Grid
          item
          xs={12}
          md={12}
          style={{ paddingBottom: 0 }}
        >
          {job ? (
            <SimpleAccordion
              summary="Ansible Job Output"
              defaultExpanded
            >
              <Container>
                <DeploymentJobOutput deploymentJob={job} />
              </Container>
            </SimpleAccordion>
          ) : (
            <></>
          )}
        </Grid>
      </AccessControl>
    </StyledGrid>
  );
}
